#!/usr/bin/python
# -*- coding: utf-8 -*-
"""International Association for the Properties of Water and Steam (IAPWS)."""

import os

from ._iapws import (_Ice, _Sublimation_Pressure, _Melting_Pressure,  # noqa
                     _Viscosity, _ThCond, _Tension, _Dielectric, _Refractive)
from .iapws97 import IAPWS97  # noqa
from .iapws95 import IAPWS95, D2O  # noqa
from .iapws08 import SeaWater  # noqa
from .humidAir import HumidAir  # noqa
from .ammonia import H2ONH3  # noqa

basepath = os.path.abspath(os.path.dirname(__file__))
with open(os.path.join(basepath, 'VERSION')) as version_file:
    __version__ = version_file.read().strip()

__doi__ = {
    "R1-76":
        {"autor": "IAPWS",
         "title": "Revised Release on the Surface Tension of Ordynary Water "
                  "Substance",
         "ref": "2014",
         "doi": ""},
    "R2-83":
        {"autor": "IAPWS",
         "title": "Release on the Values of Temperature, Pressure and Density "
                  "of Ordynary and Heavy Water Substances at their Respectives"
                  " Critical Points",
         "ref": "1992",
         "doi": ""},
    "R4-84":
        {"autor": "IAPWS",
         "title": "Revised Release on Viscosity and Thermal Conductivity of "
                  "Heavy Water Substance",
         "ref": "2007",
         "doi": ""},
    "R5-85":
        {"autor": "IAPWS",
         "title": "Release on Surface Tension of Heavy Water Substance",
         "ref": "1994",
         "doi": ""},
    "R6-95":
        {"autor": "IAPWS",
         "title": "Revised Release on the IAPWS Formulation 1995 for the "
                  "Thermodynamic Properties of Ordinary Water Substance for "
                  "General and Scientific Use",
         "ref": "2006",
         "doi": ""},
    "R7-97":
        {"autor": "IAPWS",
         "title": "Revised Release on the IAPWS Industrial Formulation 1997 "
                  "for the Thermodynamic Properties of Water and Steam",
         "ref": "2012",
         "doi": ""},
    "R8-97":
        {"autor": "IAPWS",
         "title": "Release on the Static Dielectric Constant of Ordinary Water"
                  "Substance for Temperatures from 238 K to 873 K and "
                  "Pressures up to 1000 MPa",
         "ref": "1997",
         "doi": ""},
    "R9-97":
        {"autor": "IAPWS",
         "title": "Release on the Refractive Index of Ordinary Water Substance"
                  " as a Function of Wavelength, Temperature and Pressure",
         "ref": "1997",
         "doi": ""},
    "R10-06":
        {"autor": "IAPWS",
         "title": "Revised Release on the Equation of State 2006 for H2O Ice "
                  "Ih",
         "ref": "2009",
         "doi": ""},
    "R11-19":
        {"autor": "IAPWS",
         "title": "Release on the Ionization Constant of H2O",
         "ref": "2019",
         "doi": ""},
    "R12-08":
        {"autor": "IAPWS",
         "title": "Release on the IAPWS Formulation 2008 for the Viscosity of "
                  "Ordinary Water Substance",
         "ref": "2008",
         "doi": ""},
    "R13-08":
        {"autor": "IAPWS",
         "title": "Release on the IAPWS Formulation 2008 for the Thermodynamic"
                  " Properties of Seawater",
         "ref": "2008",
         "doi": ""},
    "R14-08":
        {"autor": "IAPWS",
         "title": "Revised Release on the Pressure along the Melting and "
                  "Sublimation Curves of Ordinary Water Substance",
         "ref": "2011",
         "doi": ""},
    "R15-11":
        {"autor": "IAPWS",
         "title": "Release on the IAPWS Formulation 2011 for the Thermal "
                  "Conductivity of Ordinary Water Substance",
         "ref": "2011",
         "doi": ""},
    "R16-17":
        {"autor": "IAPWS",
         "title": "Release on the IAPWS Formulation 2017 for the Thermodynamic"
                  " Properties of Heavy Water",
         "ref": "2017",
         "doi": ""},
    "SR1-86":
        {"autor": "IAPWS",
         "title": "Revised Supplementary Release on Saturation Properties of "
                  "Ordinary Water Substance",
         "ref": "1992",
         "doi": ""},
    "SR2-01":
        {"autor": "IAPWS",
         "title": "Revised Supplementary Release on Backward Equations for "
                  "Pressure as a Function of Enthalpy and Entropy p(h,s) for "
                  "Regions 1 and 2 of the IAPWS Industrial Formulation 1997 "
                  "for the Thermodynamic Properties of Water and Steam",
         "ref": "2014",
         "doi": ""},
    "SR3-03":
        {"autor": "IAPWS",
         "title": "Revised Supplementary Release on Backward Equations for "
                  "the Functions T(p,h), v(p,h), and T(p,s), v(p,s) for "
                  "Region 3 of the IAPWS Industrial Formulation 1997 for the "
                  "Thermodynamic Properties of Water and Steam",
         "ref": "2014",
         "doi": ""},
    "SR4-04":
        {"autor": "IAPWS",
         "title": "Revised Supplementary Release on Backward Equations p(h,s) "
                  "for Region 3, Equations as a Function of h and s for the "
                  "Region Boundaries, and an Equation Tsat(h,s) for Region 4 "
                  "of the IAPWS Industrial Formulation 1997 for the "
                  "Thermodynamic Properties of Water and Steam",
         "ref": "2014",
         "doi": ""},
    "SR5-05":
        {"autor": "IAPWS",
         "title": "Revised Supplementary Release on Backward Equations for "
                  "Specific Volume as a Function of Pressure and Temperature "
                  "v(p,T) for Region 3 of the IAPWS Industrial Formulation "
                  "1997 for the Thermodynamic Properties of Water and Steam",
         "ref": "2016",
         "doi": ""},
    "SR6-08":
        {"autor": "IAPWS",
         "title": "Revised Supplementary Release on Properties of Liquid "
                  "Water at 0.1 MPa",
         "ref": "2011",
         "doi": ""},

    "SR7-09":
        {"autor": "IAPWS",
         "title": "Supplementary Release on a Computationally Efficient "
                  "Thermodynamic Formulation for Liquid Water for "
                  "Oceanographic Use",
         "ref": "2009",
         "doi": ""},
    "G1-90":
        {"autor": "IAPWS",
         "title": "Electrolytic Conductivity (Specific Conductance) of Liquid "
                  "and Dense Supercritical Water from 0°C to 800°C and "
                  "Pressures up to 1000 MPa",
         "ref": "1990",
         "doi": ""},
    "G2-90":
        {"autor": "IAPWS",
         "title": "Solubility of Sodium Sulfate in Aqueous Mixtures of Sodium "
                  "Chloride and Sulfuric Acid from Water to Concentrated "
                  "Solutions, from 250 °C to 350 °C",
         "ref": "1994",
         "doi": ""},
    "G3-00":
        {"autor": "IAPWS",
         "title": "Revised Guideline on the Critical Locus of Aqueous "
                  "Solutions of Sodium Chloride",
         "ref": "2012",
         "doi": ""},
    "G4-01":
        {"autor": "IAPWS",
         "title": "Guideline on the IAPWS Formulation 2001 for the "
                  "Thermodynamic Properties of Ammonia-Water Mixtures",
         "ref": "2001",
         "doi": ""},
    "G5-01":
        {"autor": "IAPWS",
         "title": "Guideline on the Use of Fundamental Physical Constants and "
                  "Basic Constants of Water",
         "ref": "2016",
         "doi": ""},
    "G7-04":
        {"autor": "IAPWS",
         "title": "Guideline on the Henry's Constant and Vapor-Liquid "
                  "Distribution Constant for Gases in H2O and D2O at High "
                  "Temperatures",
         "ref": "2004",
         "doi": ""},

    "G8-10":
        {"autor": "IAPWS",
         "title": "Guideline on an Equation of State for Humid Air in Contact "
                  "with Seawater and Ice, Consistent with the IAPWS "
                  "Formulation 2008 for the Thermodynamic Properties of "
                  "Seawater",
         "ref": "2010",
         "doi": ""},
    "G9-12":
        {"autor": "IAPWS",
         "title": "Guideline on a Low-Temperature Extension of the IAPWS-95 "
                  "Formulation for Water Vapor",
         "ref": "2012",
         "doi": ""},
    "G10-15":
        {"autor": "IAPWS",
         "title": "Guideline on the Thermal Conductivity of Seawater",
         "ref": "2015",
         "doi": ""},
    "G11-15":
        {"autor": "IAPWS",
         "title": "Guideline on a Virial Equation for the Fugacity of H2O in "
                  "Humid Air",
         "ref": "2015",
         "doi": ""},
    "G12-15":
        {"autor": "IAPWS",
         "title": "Guideline on Thermodynamic Properties of Supercooled Water",
         "ref": "2015",
         "doi": ""},
    "AN3-07":
        {"autor": "IAPWS",
         "title": "Thermodynamic Derivatives from IAPWS Formulations",
         "ref": "2014",
         "doi": ""},
    "AN5-13":
        {"autor": "IAPWS",
         "title": "Industrial Calculation of the Thermodynamic Properties of "
                  "Seawater",
         "ref": "2016",
         "doi": ""},
}



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""
IAPWS standard for Seawater IAPWS08 and related functionality. The module
include:

:class:`SeaWater`: Global module class with all the functionality integrated

Other functionality:
   * :func:`_Tb`: Boiling temperature of seawater
   * :func:`_Tf`: Freezing temperature of seawater
   * :func:`_Triple`: Triple point properties of seawater
   * :func:`_OsmoticPressure`: Osmotic pressure of seawater
   * :func:`_ThCond_SeaWater`: Thermal conductivity of seawater
   * :func:`_Tension_SeaWater`: Surface tension of seawater
   * :func:`_solNa2SO4`: Solubility of sodium sulfate in aqueous mixtures of
     sodium chloride and sulfuric acid
   * :func:`_critNaCl`: Critical locus of aqueous solutions of sodium chloride
"""

from __future__ import division
from math import exp, log
import warnings

from scipy.optimize import fsolve

from .iapws95 import IAPWS95
from .iapws97 import IAPWS97, _Region1, _Region2
from ._iapws import _ThCond, Tc, Pc, rhoc, _Ice, _Tension
from ._utils import deriv_G


# Constants
Rm = 8.314472
Sn = 0.03516504
S_ = Sn*40/35
Ms = 31.4038218
T_ = 40
P_ = 100
Po = 0.101325
To = 273.15


class SeaWater(object):
    """
    Class to model seawater with standard IAPWS-08

    Parameters
    ----------
    T : float
        Temperature, [K]
    P : float
        Pressure, [MPa]
    S : float
        Salinity, [kg/kg]

    fast : bool, default False
        Use the Supplementary release SR7-09 to speed up the calculation
    IF97 : bool, default False
        Use the Advisory Note No. 5 with industrial formulation

    Returns
    -------
    rho : float
        Density, [kg/m³]
    v : float
        Specific volume, [m³/kg]
    h : float
        Specific enthalpy, [kJ/kg]
    s : float
        Specific entropy, [kJ/kg·K]
    u : float
        Specific internal energy, [kJ/kg]
    g : float
        Specific Gibbs free energy, [kJ/kg]
    a : float
        Specific Helmholtz free energy, [kJ/kg]
    cp : float
        Specific isobaric heat capacity, [kJ/kg·K]
    cv : float
        Specific isochoric heat capacity, [kJ/kg·K]
    gt : float
        Derivative Gibbs energy with temperature, [kJ/kg·K]
    gp : float
        Derivative Gibbs energy with pressure, [m³/kg]
    gtt : float
        Derivative Gibbs energy with temperature square, [kJ/kg·K²]
    gtp : float
        Derivative Gibbs energy with pressure and temperature, [m³/kg·K]
    gpp : float
        Derivative Gibbs energy with temperature square, [m³/kg·MPa]
    gs : float
        Derivative Gibbs energy with salinity, [kJ/kg]
    gsp : float
        Derivative Gibbs energy with salinity and pressure, [m³/kg]
    alfav : float
        Thermal expansion coefficient, [1/K]
    betas : float
        Isentropic temperature-pressure coefficient, [K/MPa]
    xkappa : float
        Isothermal compressibility, [1/MPa]
    ks : float
        Isentropic compressibility, [1/MPa]
    w : float
        Sound Speed, [m/s]

    k : float
        Thermal conductivity, [W/m·K]
    sigma: float
        Surface tension, [N/m]

    m : float
        Molality of seawater, [mol/kg]
    mu : float
        Relative chemical potential, [kJ/kg]
    muw : float
        Chemical potential of H2O, [kJ/kg]
    mus : float
        Chemical potential of sea salt, [kJ/kg]
    osm : float
        Osmotic coefficient, [-]
    haline : float
        Haline contraction coefficient, [kg/kg]

    Notes
    -----
    :class:`Warning` if input isn't in limit:

        * 261 ≤ T ≤ 353
        * 0 < P ≤ 100
        * 0 ≤ S ≤ 0.12

    References
    ----------
    IAPWS, Release on the IAPWS Formulation 2008 for the Thermodynamic
    Properties of Seawater, http://www.iapws.org/relguide/Seawater.html

    IAPWS, Supplementary Release on a Computationally Efficient Thermodynamic
    Formulation for Liquid Water for Oceanographic Use,
    http://www.iapws.org/relguide/OceanLiquid.html

    IAPWS, Guideline on the Thermal Conductivity of Seawater,
    http://www.iapws.org/relguide/Seawater-ThCond.html

    IAPWS, Guideline on the Surface Tension of Seawater,
    http://www.iapws.org/relguide/Seawater-Surf.html

    IAPWS, Revised Advisory Note No. 3: Thermodynamic Derivatives from IAPWS
    Formulations, http://www.iapws.org/relguide/Advise3.pdf

    IAPWS,  Advisory Note No. 5: Industrial Calculation of the Thermodynamic
    Properties of Seawater, http://www.iapws.org/relguide/Advise5.html

    Examples
    --------
    >>> salt = iapws.SeaWater(T=300, P=1, S=0.04)
    >>> salt.rho
    1026.7785717245113
    >>> salt.gs
    88.56221805501536
    >>> salt.haline
    0.7311487666026304
    """

    kwargs = {"T": 0.0,
              "P": 0.0,
              "S": None,
              "fast": False,
              "IF97": False}
    status = 0
    msg = "Undefined"

    def __init__(self, **kwargs):
        """Constructor, initinialice kwargs"""
        self.kwargs = SeaWater.kwargs.copy()
        self.__call__(**kwargs)

    def __call__(self, **kwargs):
        """Make instance callable to can add input parameter one to one"""
        self.kwargs.update(kwargs)

        if self.kwargs["T"] and self.kwargs["P"] and \
                self.kwargs["S"] is not None:
            self.status = 1
            self.calculo()
            self.msg = ""

    def calculo(self):
        """Calculate procedure"""
        T = self.kwargs["T"]
        P = self.kwargs["P"]
        S = self.kwargs["S"]

        self.m = S/(1-S)/Ms
        if self.kwargs["fast"] and T <= 313.15:
            pw = self._waterSupp(T, P)
        elif self.kwargs["IF97"]:
            pw = self._waterIF97(T, P)
        else:
            pw = self._water(T, P)
        ps = self._saline(T, P, S)

        prop = {}
        for key in ps:
            prop[key] = pw[key]+ps[key]
            self.__setattr__(key, prop[key])

        self.T = T
        self.P = P
        self.rho = 1./prop["gp"]
        self.v = prop["gp"]
        self.s = -prop["gt"]
        self.cp = -T*prop["gtt"]
        self.cv = T*(prop["gtp"]**2/prop["gpp"]-prop["gtt"])
        self.h = prop["g"]-T*prop["gt"]
        self.u = prop["g"]-T*prop["gt"]-P*1000*prop["gp"]
        self.a = prop["g"]-P*1000*prop["gp"]
        self.alfav = prop["gtp"]/prop["gp"]
        self.betas = -prop["gtp"]/prop["gtt"]
        self.xkappa = -prop["gpp"]/prop["gp"]
        self.ks = (prop["gtp"]**2-prop["gtt"]*prop["gpp"])/prop["gp"] / \
            prop["gtt"]
        self.w = prop["gp"]*(prop["gtt"]*1000/(
            prop["gtp"]**2 - prop["gtt"]*1000*prop["gpp"]*1e-6))**0.5

        # Thermal conductivity calculation
        if "thcond" in pw:
            kw = pw["thcond"]
        else:
            kw = _ThCond(1/pw["gp"], T)
        try:
            self.k = _ThCond_SeaWater(T, P, S)+kw
        except NotImplementedError:
            self.k = None

        # Surface tension calculation
        try:
            self.sigma = _Tension_SeaWater(T, S)
        except NotImplementedError:
            self.sigma = None

        if S:
            self.mu = prop["gs"]
            self.muw = prop["g"]-S*prop["gs"]
            self.mus = prop["g"]+(1-S)*prop["gs"]
            self.osm = -(ps["g"]-S*prop["gs"])/self.m/Rm/T
            self.haline = -prop["gsp"]/prop["gp"]
        else:
            self.mu = None
            self.muw = None
            self.mus = None
            self.osm = None
            self.haline = None

    def derivative(self, z, x, y):
        """
        Wrapper derivative for custom derived properties
        where x, y, z can be: P, T, v, u, h, s, g, a
        """
        return deriv_G(self, z, x, y, self)

    @classmethod
    def _water(cls, T, P):
        """Get properties of pure water, Table4 pag 8"""
        water = IAPWS95(P=P, T=T)
        prop = {}
        prop["g"] = water.h-T*water.s
        prop["gt"] = -water.s
        prop["gp"] = 1./water.rho
        prop["gtt"] = -water.cp/T
        prop["gtp"] = water.betas*water.cp/T
        prop["gpp"] = -1e6/(water.rho*water.w)**2-water.betas**2*1e3*water.cp/T
        prop["gs"] = 0
        prop["gsp"] = 0
        prop["thcond"] = water.k
        return prop

    @classmethod
    def _waterIF97(cls, T, P):
        water = IAPWS97(P=P, T=T)
        betas = water.derivative("T", "P", "s", water)
        prop = {}
        prop["g"] = water.h-T*water.s
        prop["gt"] = -water.s
        prop["gp"] = 1./water.rho
        prop["gtt"] = -water.cp/T
        prop["gtp"] = betas*water.cp/T
        prop["gpp"] = -1e6/(water.rho*water.w)**2-betas**2*1e3*water.cp/T
        prop["gs"] = 0
        prop["gsp"] = 0
        return prop

    @classmethod
    def _waterSupp(cls, T, P):
        """
        Get properties of pure water using the supplementary release SR7-09,
        Table4 pag 6
        """
        tau = (T-273.15)/40
        pi = (P-0.101325)/100

        J = [0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 3, 3, 3,
             3, 3, 3, 4, 4, 4, 4, 4, 5, 5, 5, 5, 5, 6, 6, 6, 6, 7, 7]
        K = [0, 1, 2, 3, 4, 5, 6, 0, 1, 2, 3, 4, 5, 0, 1, 2, 3, 4, 5, 0, 1, 2,
             3, 4, 5, 0, 1, 2, 3, 4, 0, 1, 2, 3, 4, 0, 1, 2, 3, 0, 1]
        G = [0.101342743139674e3, 0.100015695367145e6, -0.254457654203630e4,
             0.284517778446287e3, -0.333146754253611e2, 0.420263108803084e1,
             -0.546428511471039, 0.590578347909402e1, -0.270983805184062e3,
             0.776153611613101e3, -0.196512550881220e3, 0.289796526294175e2,
             -0.213290083518327e1, -0.123577859330390e5, 0.145503645404680e4,
             -0.756558385769359e3, 0.273479662323528e3, -0.555604063817218e2,
             0.434420671917197e1, 0.736741204151612e3, -0.672507783145070e3,
             0.499360390819152e3, -0.239545330654412e3, 0.488012518593872e2,
             -0.166307106208905e1, -0.148185936433658e3, 0.397968445406972e3,
             -0.301815380621876e3, 0.152196371733841e3, -0.263748377232802e2,
             0.580259125842571e2, -0.194618310617595e3, 0.120520654902025e3,
             -0.552723052340152e2, 0.648190668077221e1, -0.189843846514172e2,
             0.635113936641785e2, -0.222897317140459e2, 0.817060541818112e1,
             0.305081646487967e1, -0.963108119393062e1]

        g, gt, gp, gtt, gtp, gpp = 0, 0, 0, 0, 0, 0
        for j, k, gi in zip(J, K, G):
            g += gi*tau**j*pi**k
            if j >= 1:
                gt += gi*j*tau**(j-1)*pi**k
            if k >= 1:
                gp += k*gi*tau**j*pi**(k-1)
            if j >= 2:
                gtt += j*(j-1)*gi*tau**(j-2)*pi**k
            if j >= 1 and k >= 1:
                gtp += j*k*gi*tau**(j-1)*pi**(k-1)
            if k >= 2:
                gpp += k*(k-1)*gi*tau**j*pi**(k-2)

        prop = {}
        prop["g"] = g*1e-3
        prop["gt"] = gt/40*1e-3
        prop["gp"] = gp/100*1e-6
        prop["gtt"] = gtt/40**2*1e-3
        prop["gtp"] = gtp/40/100*1e-6
        prop["gpp"] = gpp/100**2*1e-6
        prop["gs"] = 0
        prop["gsp"] = 0
        return prop

    @classmethod
    def _saline(cls, T, P, S):
        """Eq 4"""
        # Check input in range of validity
        if T <= 261 or T > 353 or P <= 0 or P > 100 or S < 0 or S > 0.12:
            warnings.warn("Incoming out of bound")

        S_ = 0.03516504*40/35
        X = (S/S_)**0.5
        tau = (T-273.15)/40
        pi = (P-0.101325)/100

        Li = [1, 2, 3, 4, 5, 6, 7, 1, 2, 3, 4, 5, 6, 2, 3, 4, 2, 3, 4, 2, 3, 4,
              2, 4, 2, 2, 3, 4, 5, 2, 3, 4, 2, 3, 2, 3, 2, 3, 2, 3, 4, 2, 3, 2,
              3, 2, 2, 2, 3, 4, 2, 3, 2, 3, 2, 2, 2, 3, 2, 2, 2, 2, 2, 2]
        Lj = [0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 2, 2, 2, 3, 3, 3, 4, 4, 4,
              5, 5, 6, 0, 0, 0, 0, 1, 1, 1, 2, 2, 3, 3, 4, 4, 0, 0, 0, 1, 1, 2,
              2, 3, 4, 0, 0, 0, 1, 1, 2, 2, 3, 4, 0, 0, 1, 2, 3, 0, 1, 2]
        Lk = [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
              0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2,
              2, 2, 2, 3, 3, 3, 3, 3, 3, 3, 3, 3, 4, 4, 4, 4, 4, 5, 5, 5]
        G = [0.581281456626732e4, 0.141627648484197e4, -0.243214662381794e4,
             0.202580115603697e4, -0.109166841042967e4, 0.374601237877840e3,
             -0.485891069025409e2, 0.851226734946706e3, 0.168072408311545e3,
             -0.493407510141682e3, 0.543835333000098e3, -0.196028306689776e3,
             0.367571622995805e2, 0.880031352997204e3, -0.430664675978042e2,
             -0.685572509204491e2, -0.225267649263401e3, -0.100227370861875e2,
             0.493667694856254e2, 0.914260447751259e2, 0.875600661808945,
             -0.171397577419788e2, -0.216603240875311e2, 0.249697009569508e1,
             0.213016970847183e1, -0.331049154044839e4, 0.199459603073901e3,
             -0.547919133532887e2, 0.360284195611086e2, 0.729116529735046e3,
             -0.175292041186547e3, -0.226683558512829e2, -0.860764303783977e3,
             0.383058066002476e3, 0.694244814133268e3, -0.460319931801257e3,
             -0.297728741987187e3, 0.234565187611355e3, 0.384794152978599e3,
             -0.522940909281335e2, -0.408193978912261e1, -0.343956902961561e3,
             0.831923927801819e2, 0.337409530269367e3, -0.541917262517112e2,
             -0.204889641964903e3, 0.747261411387560e2, -0.965324320107458e2,
             0.680444942726459e2, -0.301755111971161e2, 0.124687671116248e3,
             -0.294830643494290e2, -0.178314556207638e3, 0.256398487389914e2,
             0.113561697840594e3, -0.364872919001588e2, 0.158408172766824e2,
             -0.341251932441282e1, -0.316569643860730e2, 0.442040358308000e2,
             -0.111282734326413e2, -0.262480156590992e1, 0.704658803315449e1,
             -0.792001547211682e1]

        g, gt, gp, gtt, gtp, gpp, gs, gsp = 0, 0, 0, 0, 0, 0, 0, 0

        # Calculate only for some salinity
        if S != 0:
            for i, j, k, gi in zip(Li, Lj, Lk, G):
                if i == 1:
                    g += gi*X**2*log(X)*tau**j*pi**k
                    gs += gi*(2*log(X)+1)*tau**j*pi**k
                else:
                    g += gi*X**i*tau**j*pi**k
                    gs += i*gi*X**(i-2)*tau**j*pi**k
                if j >= 1:
                    if i == 1:
                        gt += gi*X**2*log(X)*j*tau**(j-1)*pi**k
                    else:
                        gt += gi*X**i*j*tau**(j-1)*pi**k
                if k >= 1:
                    gp += k*gi*X**i*tau**j*pi**(k-1)
                    gsp += i*k*gi*X**(i-2)*tau**j*pi**(k-1)
                if j >= 2:
                    gtt += j*(j-1)*gi*X**i*tau**(j-2)*pi**k
                if j >= 1 and k >= 1:
                    gtp += j*k*gi*X**i*tau**(j-1)*pi**(k-1)
                if k >= 2:
                    gpp += k*(k-1)*gi*X**i*tau**j*pi**(k-2)

        prop = {}
        prop["g"] = g*1e-3
        prop["gt"] = gt/40*1e-3
        prop["gp"] = gp/100*1e-6
        prop["gtt"] = gtt/40**2*1e-3
        prop["gtp"] = gtp/40/100*1e-6
        prop["gpp"] = gpp/100**2*1e-6
        prop["gs"] = gs/S_/2*1e-3
        prop["gsp"] = gsp/S_/2/100*1e-6
        return prop


def _Tb(P, S):
    """Procedure to calculate the boiling temperature of seawater

    Parameters
    ----------
    P : float
        Pressure, [MPa]
    S : float
        Salinity, [kg/kg]

    Returns
    -------
    Tb : float
        Boiling temperature, [K]

    References
    ----------
    IAPWS,  Advisory Note No. 5: Industrial Calculation of the Thermodynamic
    Properties of Seawater, http://www.iapws.org/relguide/Advise5.html, Eq 7
    """
    def f(T):
        pw = _Region1(T, P)
        gw = pw["h"]-T*pw["s"]

        pv = _Region2(T, P)
        gv = pv["h"]-T*pv["s"]

        ps = SeaWater._saline(T, P, S)
        return -ps["g"]+S*ps["gs"]-gw+gv

    Tb = fsolve(f, 300)[0]
    return Tb


def _Tf(P, S):
    """Procedure to calculate the freezing temperature of seawater

    Parameters
    ----------
    P : float
        Pressure, [MPa]
    S : float
        Salinity, [kg/kg]

    Returns
    -------
    Tf : float
        Freezing temperature, [K]

    References
    ----------
    IAPWS,  Advisory Note No. 5: Industrial Calculation of the Thermodynamic
    Properties of Seawater, http://www.iapws.org/relguide/Advise5.html, Eq 12
    """
    def f(T):
        T = float(T)
        pw = _Region1(T, P)
        gw = pw["h"]-T*pw["s"]

        gih = _Ice(T, P)["g"]

        ps = SeaWater._saline(T, P, S)
        return -ps["g"]+S*ps["gs"]-gw+gih

    Tf = fsolve(f, 300)[0]
    return Tf


def _Triple(S):
    """Procedure to calculate the triple point pressure and temperature for
    seawater

    Parameters
    ----------
    S : float
        Salinity, [kg/kg]

    Returns
    -------
    prop : dict
        Dictionary with the triple point properties:

            * Tt: Triple point temperature, [K]
            * Pt: Triple point pressure, [MPa]

    References
    ----------
    IAPWS,  Advisory Note No. 5: Industrial Calculation of the Thermodynamic
    Properties of Seawater, http://www.iapws.org/relguide/Advise5.html, Eq 7
    """
    def f(parr):
        T, P = parr
        pw = _Region1(T, P)
        gw = pw["h"]-T*pw["s"]

        pv = _Region2(T, P)
        gv = pv["h"]-T*pv["s"]

        gih = _Ice(T, P)["g"]
        ps = SeaWater._saline(T, P, S)

        return -ps["g"]+S*ps["gs"]-gw+gih, -ps["g"]+S*ps["gs"]-gw+gv

    Tt, Pt = fsolve(f, [273, 6e-4])

    prop = {}
    prop["Tt"] = Tt
    prop["Pt"] = Pt
    return prop


def _OsmoticPressure(T, P, S):
    """Procedure to calculate the osmotic pressure of seawater

    Parameters
    ----------
    T : float
        Tmperature, [K]
    P : float
        Pressure, [MPa]
    S : float
        Salinity, [kg/kg]

    Returns
    -------
    Posm : float
        Osmotic pressure, [MPa]

    References
    ----------
    IAPWS,  Advisory Note No. 5: Industrial Calculation of the Thermodynamic
    Properties of Seawater, http://www.iapws.org/relguide/Advise5.html, Eq 15
    """
    pw = _Region1(T, P)
    gw = pw["h"]-T*pw["s"]

    def f(Posm):
        pw2 = _Region1(T, P+Posm)
        gw2 = pw2["h"]-T*pw2["s"]
        ps = SeaWater._saline(T, P+Posm, S)
        return -ps["g"]+S*ps["gs"]-gw+gw2

    Posm = fsolve(f, 0)[0]
    return Posm


def _ThCond_SeaWater(T, P, S):
    """Equation for the thermal conductivity of seawater

    Parameters
    ----------
    T : float
        Temperature, [K]
    P : float
        Pressure, [MPa]
    S : float
        Salinity, [kg/kg]

    Returns
    -------
    k : float
        Thermal conductivity excess relative to that of the pure water, [W/mK]

    Notes
    -----
    Raise :class:`NotImplementedError` if input isn't in limit:

        * 273.15 ≤ T ≤ 523.15
        * 0 ≤ P ≤ 140
        * 0 ≤ S ≤ 0.17

    Examples
    --------
    >>> _ThCond_Seawater(293.15, 0.1, 0.035)
    -0.00418604

    References
    ----------
    IAPWS, Guideline on the Thermal Conductivity of Seawater,
    http://www.iapws.org/relguide/Seawater-ThCond.html
    """
    # Check input parameters
    if T < 273.15 or T > 523.15 or P < 0 or P > 140 or S < 0 or S > 0.17:
        raise NotImplementedError("Incoming out of bound")

    # Eq 4
    a1 = -7.180891e-5+1.831971e-7*P
    a2 = 1.048077e-3-4.494722e-6*P

    # Eq 5
    b1 = 1.463375e-1+9.208586e-4*P
    b2 = -3.086908e-3+1.798489e-5*P

    a = a1*exp(a2*(T-273.15))  # Eq 2
    b = b1*exp(b2*(T-273.15))  # Eq 3

    # Eq 1
    DL = a*(1000*S)**(1+b)
    return DL


def _Tension_SeaWater(T, S):
    """Equation for the surface tension of seawater

    Parameters
    ----------
    T : float
        Temperature, [K]
    S : float
        Salinity, [kg/kg]

    Returns
    -------
    σ : float
        Surface tension, [N/m]

    Notes
    -----
    Raise :class:`NotImplementedError` if input isn't in limit:

        * 0 ≤ S ≤ 0.131  for 274.15 ≤ T ≤ 365.15
        * 0 ≤ S ≤ 0.038  for 248.15 ≤ T ≤ 274.15

    Examples
    --------
    >>> _Tension_Seawater(253.15, 0.035)
    -0.07922517961

    References
    ----------
    IAPWS, Guideline on the Surface Tension of Seawater,
    http://www.iapws.org/relguide/Seawater-Surf.html
    """
    # Check input parameters
    if 248.15 < T < 274.15:
        if S < 0 or S > 0.038:
            raise NotImplementedError("Incoming out of bound")
    elif 274.15 < T < 365.15:
        if S < 0 or S > 0.131:
            raise NotImplementedError("Incoming out of bound")
    else:
        raise NotImplementedError("Incoming out of bound")

    sw = _Tension(T)
    sigma = sw*(1+3.766e-1*S+2.347e-3*S*(T-273.15))
    return sigma


def _solNa2SO4(T, mH2SO4, mNaCl):
    """Equation for the solubility of sodium sulfate in aqueous mixtures of
    sodium chloride and sulfuric acid

    Parameters
    ----------
    T : float
        Temperature, [K]
    mH2SO4 : float
        Molality of sufuric acid, [mol/kg(water)]
    mNaCl : float
        Molality of sodium chloride, [mol/kg(water)]

    Returns
    -------
    S : float
        Molal solutility of sodium sulfate, [mol/kg(water)]

    Notes
    -----
    Raise :class:`NotImplementedError` if input isn't in limit:

        * 523.15 ≤ T ≤ 623.15
        * 0 ≤ mH2SO4 ≤ 0.75
        * 0 ≤ mNaCl ≤ 2.25

    Examples
    --------
    >>> _solNa2SO4(523.15, 0.25, 0.75)
    2.68

    References
    ----------
    IAPWS, Solubility of Sodium Sulfate in Aqueous Mixtures of Sodium Chloride
    and Sulfuric Acid from Water to Concentrated Solutions,
    http://www.iapws.org/relguide/na2so4.pdf
    """
    # Check input parameters
    if T < 523.15 or T > 623.15 or mH2SO4 < 0 or mH2SO4 > 0.75 or \
            mNaCl < 0 or mNaCl > 2.25:
        raise NotImplementedError("Incoming out of bound")

    A00 = -0.8085987*T+81.4613752+0.10537803*T*log(T)
    A10 = 3.4636364*T-281.63322-0.46779874*T*log(T)
    A20 = -6.0029634*T+480.60108+0.81382854*T*log(T)
    A30 = 4.4540258*T-359.36872-0.60306734*T*log(T)
    A01 = 0.4909061*T-46.556271-0.064612393*T*log(T)
    A02 = -0.002781314*T+1.722695+0.0000013319698*T*log(T)
    A03 = -0.014074108*T+0.99020227+0.0019397832*T*log(T)
    A11 = -0.87146573*T+71.808756+0.11749585*T*log(T)

    S = A00 + A10*mH2SO4 + A20*mH2SO4**2 + A30*mH2SO4**3 + A01*mNaCl + \
        A02*mNaCl**2 + A03*mNaCl**3 + A11*mH2SO4*mNaCl

    return S


def _critNaCl(x):
    """Equation for the critical locus of aqueous solutions of sodium chloride

    Parameters
    ----------
    x : float
        Mole fraction of NaCl, [-]

    Returns
    -------
    prop : dict
        A dictionary withe the properties:

            * Tc: critical temperature, [K]
            * Pc: critical pressure, [MPa]
            * rhoc: critical density, [kg/m³]

    Notes
    -----
    Raise :class:`NotImplementedError` if input isn't in limit:

        * 0 ≤ x ≤ 0.12

    Examples
    --------
    >>> _critNaCl(0.1)
    975.571016

    References
    ----------
    IAPWS, Revised Guideline on the Critical Locus of Aqueous Solutions of
    Sodium Chloride, http://www.iapws.org/relguide/critnacl.html
    """
    # Check input parameters
    if x < 0 or x > 0.12:
        raise NotImplementedError("Incoming out of bound")

    T1 = Tc*(1 + 2.3e1*x - 3.3e2*x**1.5 - 1.8e3*x**2)
    T2 = Tc*(1 + 1.757e1*x - 3.026e2*x**1.5 + 2.838e3*x**2 - 1.349e4*x**2.5
             + 3.278e4*x**3 - 3.674e4*x**3.5 + 1.437e4*x**4)
    f1 = (abs(10000*x-10-1)-abs(10000*x-10+1))/4+0.5
    f2 = (abs(10000*x-10+1)-abs(10000*x-10-1))/4+0.5

    # Eq 1
    tc = f1*T1+f2*T2

    # Eq 7
    rc = rhoc*(1 + 1.7607e2*x - 2.9693e3*x**1.5 + 2.4886e4*x**2
               - 1.1377e5*x**2.5 + 2.8847e5*x**3 - 3.8195e5*x**3.5
               + 2.0633e5*x**4)

    # Eq 8
    DT = tc-Tc
    pc = Pc*(1+9.1443e-3*DT+5.1636e-5*DT**2-2.5360e-7*DT**3+3.6494e-10*DT**4)

    prop = {}
    prop["Tc"] = tc
    prop["rhoc"] = rc
    prop["Pc"] = pc
    return prop



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""
Module with Ammonia-water mixture properties and related properties. The module
include:

    * :class:`NH3`: Multiparameter equation of state for ammonia
    * :class:`H2ONH3`:  Thermodynamic properties of ammonia-water mixtures
    * :class:`Ttr`: Triple point of ammonia-water mixtures
"""


from __future__ import division
from math import exp, log, pi
import warnings

from scipy.constants import Boltzmann
from .iapws95 import MEoS, IAPWS95, mainClassDoc


@mainClassDoc()
class NH3(MEoS):
    """Multiparameter equation of state for ammonia
    for internal procedures, see MEoS base class

    References
    ----------
    Baehr, H.D., Tillner-Roth, R.; Thermodynamic Properties of Environmentally
    Acceptable Refrigerants: Equations of State and Tables for Ammonia, R22,
    R134a, R152a, and R123. Springer-Verlag, Berlin, 1994.
    http://doi.org/10.1007/978-3-642-79400-1
    """

    name = "ammonia"
    CASNumber = "7664-41-7"
    formula = "NH3"
    synonym = "R-717"
    rhoc = 225.
    Tc = 405.40
    Pc = 11.333   # MPa
    M = 17.03026  # g/mol
    Tt = 195.495
    Tb = 239.823
    f_acent = 0.25601
    momentoDipolar = 1.470

    Fi0 = {"ao_log": [1, -1],
           "pow": [0, 1, 1./3, -1.5, -1.75],
           "ao_pow": [-15.81502, 4.255726, 11.47434, -1.296211, 0.5706757],
           "ao_exp": [], "titao": [],
           "ao_hyp": [], "hyp": []}

    _constants = {
        "R": 8.314471,

        "nr1": [-0.1858814e01, 0.4554431e-1, 0.7238548, 0.1229470e-1,
                0.2141882e-10],
        "d1": [1, 2, 1, 4, 15],
        "t1": [1.5, -0.5, 0.5, 1., 3.],

        "nr2": [-0.1430020e-1, 0.3441324, -0.2873571, 0.2352589e-4,
                -0.3497111e-1, 0.1831117e-2, 0.2397852e-1, -0.4085375e-1,
                0.2379275, -0.3548972e-1, -0.1823729, 0.2281556e-1,
                -0.6663444e-2, -0.8847486e-2, 0.2272635e-2, -0.5588655e-3],
        "d2": [3, 3, 1, 8, 2, 8, 1, 1, 2, 3, 2, 4, 3, 1, 2, 4],
        "t2": [0, 3, 4, 4, 5, 5, 3, 6, 8, 8, 10, 10, 5, 7.5, 15, 30],
        "c2": [1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 3, 3, 3, 3],
        "gamma2": [1]*16}

    _melting = {"eq": 1, "Tref": Tt, "Pref": 1000,
                "Tmin": Tt, "Tmax": 700.0,
                "a1": [], "exp1": [], "a2": [], "exp2": [],
                "a3": [0.2533125e4], "exp3": [1]}

    _surf = {"sigma": [0.1028, -0.09453], "exp": [1.211, 5.585]}
    _Pv = {
        "eq": 5,
        "ao": [-0.70993e1, -0.24330e1, 0.87591e1, -0.64091e1, -0.21185e1],
        "exp": [1., 1.5, 1.7, 1.95, 4.2]}
    _rhoL = {
        "eq": 1,
        "ao": [0.34488e2, -0.12849e3, 0.17382e3, -0.10699e3, 0.30339e2],
        "exp": [0.58, 0.75, 0.9, 1.1, 1.3]}
    _rhoG = {
        "eq": 3,
        "ao": [-.38435, -4.0846, -6.6634, -0.31881e2, 0.21306e3, -0.24648e3],
        "exp": [0.218, 0.55, 1.5, 3.7, 5.5, 5.8]}

    def _visco(self, rho, T, fase=None):
        """Equation for the Viscosity

        Parameters
        ----------
        rho : float
            Density [kg/m³]
        T : float
            Temperature [K]

        Returns
        -------
        mu : float
            Viscosity [Pa·s]

        References
        ----------
        Fenghour, A., Wakeham, W.A., Vesovic, V., Watson, J.T.R., Millat, J.,
        and Vogel, E., The viscosity of ammonia, J. Phys. Chem. Ref. Data 24,
        1649 (1995). doi:10.1063/1.555961
        """
        ek = 386
        sigma = 0.2957

        rho = rho/self.M
        T_ = T/ek

        # Eq 4
        a = [4.99318220, -0.61122364, 0.0, 0.18535124, -0.11160946]
        omega = exp(sum(ai*log(T_)**i for i, ai in enumerate(a)))

        # Eq 2, Zero-Density Limit
        muo = 2.1357*(T*self.M)**0.5/sigma**2/omega

        # Eq 8, Viscosity virial coefficient
        cv = [-0.17999496e1, 0.46692621e2, -0.53460794e3, 0.33604074e4,
              -0.13019164e5, 0.33414230e5, -0.58711743e5, 0.71426686e5,
              -0.59834012e5, 0.33652741e5, -0.1202735e5, 0.24348205e4,
              -0.20807957e3]
        Bn = 0.6022137*sigma**3*sum(c*T_**(-i/2) for i, c in enumerate(cv))
        # Eq 7
        mub = Bn*muo*rho

        # Eq 10
        dij = [2.19664285e-1, -0.83651107e-1, 0.17366936e-2, -0.64250359e-2,
               1.67668649e-4, -1.49710093e-4, 0.77012274e-4]
        ji = [2, 4, 0, 1, 2, 3, 4]
        ii = [2, 2, 3, 3, 4, 4, 4]
        mur = sum(d/T_**j*rho**i for d, j, i in zip(dij, ji, ii))

        # Eq 1
        mu = muo + mub + mur
        return mu*1e-6

    def _thermo(self, rho, T, fase):
        """Equation for the thermal conductivity

        Parameters
        ----------
        rho : float
            Density, [kg/m³]
        T : float
            Temperature, [K]
        fase: dict
            phase properties

        Returns
        -------
        k : float
            Thermal conductivity [W/mK]

        References
        ----------
        Tufeu, R., Ivanov, D.Y., Garrabos, Y., and Le Neindre, B., Thermal
        conductivity of ammonia in a large temperature and pressure range
        including the critical region, Ber. Bunsenges. Phys. Chem., 88:422-427,
        1984. doi:10.1002/bbpc.19840880421
        """
        # The paper use a diferent rhoc value to the EoS
        rhoc = 235

        if rho == rhoc and T == self.Tc:
            warnings.warn("Thermal conductiviy undefined in critical point")
            return None

        # Eq 6
        no = [0.3589e-1, -0.1750e-3, 0.4551e-6, 0.1685e-9, -0.4828e-12]
        Lo = sum(n*T**i for i, n in enumerate(no))

        # Eq 7
        nb = [0.16207e-3, 0.12038e-5, -0.23139e-8, 0.32749e-11]
        L_ = sum(n*rho**(i+1) for i, n in enumerate(nb))

        # Critical enchancement
        t = abs(T-405.4)/405.4
        dPT = 1e5*(2.18-0.12/exp(17.8*t))
        nb = 1e-5*(2.6+1.6*t)

        DL = 1.2*Boltzmann*T**2/6/pi/nb/(1.34e-10/t**0.63*(1+t**0.5)) * \
            dPT**2 * 0.423e-8/t**1.24*(1+t**0.5/0.7)

        # Add correction for entire range of temperature, Eq 10
        DL *= exp(-36*t**2)

        X = 0.61*rhoc+16.5*log(t)
        if rho > 0.6*rhoc:
            # Eq 11
            DL *= X**2/(X**2+(rho-0.96*rhoc)**2)
        else:
            # Eq 14
            DL = X**2/(X**2+(0.6*rhoc-0.96*rhoc)**2)
            DL *= rho**2/(0.6*rhoc)**2

        # Eq 5
        k = Lo+L_+DL
        return k


class H2ONH3(object):
    """Ammonia-water mixtures."""

    # TODO: Add equilibrium routine

    def _prop(self, rho, T, x):
        """Thermodynamic properties of ammonia-water mixtures

        Parameters
        ----------
        T : float
            Temperature [K]
        rho : float
            Density [kg/m³]
        x : float
            Mole fraction of ammonia in mixture [mol/mol]

        Returns
        -------
        prop : dict
            Dictionary with thermodynamic properties of ammonia-water mixtures:

                * M: Mixture molecular mass, [g/mol]
                * P: Pressure, [MPa]
                * u: Specific internal energy, [kJ/kg]
                * s: Specific entropy, [kJ/kgK]
                * h: Specific enthalpy, [kJ/kg]
                * a: Specific Helmholtz energy, [kJ/kg]
                * g: Specific gibbs energy, [kJ/kg]
                * cv: Specific isochoric heat capacity, [kJ/kgK]
                * cp: Specific isobaric heat capacity, [kJ/kgK]
                * w: Speed of sound, [m/s]
                * fugH2O: Fugacity of water, [-]
                * fugNH3: Fugacity of ammonia, [-]

        References
        ----------
        IAPWS, Guideline on the IAPWS Formulation 2001 for the Thermodynamic
        Properties of Ammonia-Water Mixtures,
        http://www.iapws.org/relguide/nh3h2o.pdf, Table 4
        """
        # FIXME: The values are good, bad difer by 1%, a error I can find
        # In Pressure happen and only use fird

        M = (1-x)*IAPWS95.M + x*NH3.M
        R = 8.314471/M

        phio = self._phi0(rho, T, x)
        fio = phio["fio"]
        tau0 = phio["tau"]
        fiot = phio["fiot"]
        fiott = phio["fiott"]

        phir = self._phir(rho, T, x)
        fir = phir["fir"]
        tau = phir["tau"]
        delta = phir["delta"]
        firt = phir["firt"]
        firtt = phir["firtt"]
        fird = phir["fird"]
        firdd = phir["firdd"]
        firdt = phir["firdt"]
        F = phir["F"]

        prop = {}
        Z = 1 + delta*fird
        prop["M"] = M
        prop["P"] = Z*R*T*rho/1000
        prop["u"] = R*T*(tau0*fiot + tau*firt)
        prop["s"] = R*(tau0*fiot + tau*firt - fio - fir)
        prop["h"] = R*T*(1+delta*fird+tau0*fiot+tau*firt)
        prop["g"] = prop["h"]-T*prop["s"]
        prop["a"] = prop["u"]-T*prop["s"]
        cvR = -tau0**2*fiott - tau**2*firtt
        prop["cv"] = R*cvR
        prop["cp"] = R*(cvR+(1+delta*fird-delta*tau*firdt)**2
                        / (1+2*delta*fird+delta**2*firdd))
        prop["w"] = (R*T*1000*(1+2*delta*fird+delta**2*firdd
                               + (1+delta*fird-delta*tau*firdt)**2 / cvR))**0.5
        prop["fugH2O"] = Z*exp(fir+delta*fird-x*F)
        prop["fugNH3"] = Z*exp(fir+delta*fird+(1-x)*F)
        return prop

    def _phi0(self, rho, T, x):
        """Ideal gas Helmholtz energy of binary mixtures and derivatives

        Parameters
        ----------
        rho : float
            Density, [kg/m³]
        T : float
            Temperature, [K]
        x : float
            Mole fraction of ammonia in mixture, [mol/mol]

        Returns
        -------
        prop : dict
            Dictionary with ideal adimensional helmholtz energy and
            derivatives:

                * tau: the adimensional temperature variable, [-]
                * delta: the adimensional density variable, [-]
                * fio,[-]
                * fiot: [∂fio/∂τ]δ  [-]
                * fiod: [∂fio/∂δ]τ  [-]
                * fiott: [∂²fio/∂τ²]δ  [-]
                * fiodt: [∂²fio/∂τ∂δ]  [-]
                * fiodd: [∂²fio/∂δ²]τ  [-]

        References
        ----------
        IAPWS, Guideline on the IAPWS Formulation 2001 for the Thermodynamic
        Properties of Ammonia-Water Mixtures,
        http://www.iapws.org/relguide/nh3h2o.pdf, Eq 2
        """
        # Define reducing parameters for mixture model
        M = (1-x)*IAPWS95.M + x*NH3.M
        tau = 500/T
        delta = rho/15/M

        # Table 2
        Fi0 = {
            "log_water": 3.006320,
            "ao_water": [-7.720435, 8.649358],
            "pow_water": [0, 1],
            "ao_exp": [0.012436, 0.97315, 1.279500, 0.969560, 0.248730],
            "titao": [1.666, 4.578, 10.018, 11.964, 35.600],
            "log_nh3": -1.0,
            "ao_nh3": [-16.444285, 4.036946, 10.69955, -1.775436, 0.82374034],
            "pow_nh3": [0, 1, 1/3, -3/2, -7/4]}

        fiod = 1/delta
        fiodd = -1/delta**2
        fiodt = 0
        fiow = fiotw = fiottw = 0
        fioa = fiota = fiotta = 0

        # Water section
        if x < 1:
            fiow = Fi0["log_water"]*log(tau) + log(1-x)
            fiotw = Fi0["log_water"]/tau
            fiottw = -Fi0["log_water"]/tau**2
            for n, t in zip(Fi0["ao_water"], Fi0["pow_water"]):
                fiow += n*tau**t
                if t != 0:
                    fiotw += t*n*tau**(t-1)
                if t not in [0, 1]:
                    fiottw += n*t*(t-1)*tau**(t-2)
            for n, t in zip(Fi0["ao_exp"], Fi0["titao"]):
                fiow += n*log(1-exp(-tau*t))
                fiotw += n*t*((1-exp(-t*tau))**-1-1)
                fiottw -= n*t**2*exp(-t*tau)*(1-exp(-t*tau))**-2

        # ammonia section
        if x > 0:
            fioa = Fi0["log_nh3"]*log(tau) + log(x)
            fiota = Fi0["log_nh3"]/tau
            fiotta = -Fi0["log_nh3"]/tau**2
            for n, t in zip(Fi0["ao_nh3"], Fi0["pow_nh3"]):
                fioa += n*tau**t
                if t != 0:
                    fiota += t*n*tau**(t-1)
                if t not in [0, 1]:
                    fiotta += n*t*(t-1)*tau**(t-2)

        prop = {}
        prop["tau"] = tau
        prop["delta"] = delta
        prop["fio"] = log(delta) + (1-x)*fiow + x*fioa
        prop["fiot"] = (1-x)*fiotw + x*fiota
        prop["fiott"] = (1-x)*fiottw + x*fiotta
        prop["fiod"] = fiod
        prop["fiodd"] = fiodd
        prop["fiodt"] = fiodt
        return prop

    def _phir(self, rho, T, x):
        """Residual contribution to the free Helmholtz energy

        Parameters
        ----------
        rho : float
            Density, [kg/m³]
        T : float
            Temperature, [K]
        x : float
            Mole fraction of ammonia in mixture, [mol/mol]

        Returns
        -------
        prop : dict
            dictionary with residual adimensional helmholtz energy and
            derivatives:

                * tau: the adimensional temperature variable, [-]
                * delta: the adimensional density variable, [-]
                * fir, [-]
                * firt: [∂fir/∂τ]δ,x  [-]
                * fird: [∂fir/∂δ]τ,x  [-]
                * firtt: [∂²fir/∂τ²]δ,x  [-]
                * firdt: [∂²fir/∂τ∂δ]x  [-]
                * firdd: [∂²fir/∂δ²]τ,x  [-]
                * firx: [∂fir/∂x]τ,δ  [-]
                * F: Function for fugacity calculation, [-]

        References
        ----------
        IAPWS, Guideline on the IAPWS Formulation 2001 for the Thermodynamic
        Properties of Ammonia-Water Mixtures,
        http://www.iapws.org/relguide/nh3h2o.pdf, Eq 3
        """
        # Temperature reducing value, Eq 4
        Tc12 = 0.9648407/2*(IAPWS95.Tc+NH3.Tc)
        Tn = (1-x)**2*IAPWS95.Tc + x**2*NH3.Tc + 2*x*(1-x**1.125455)*Tc12

        # sympy.diff("(1-x)^2*Tc1 + x^2*Tc2 + 2*x*(1-x^a)*Tc12", "x")
        # Out[]: Tc1*(2*x - 2) - 2*Tc12*a*x**a + 2*Tc12*(1 - x**a) + 2*Tc2*x
        dTnx = -2*IAPWS95.Tc*(1-x) + 2*x*NH3.Tc + 2*Tc12*(1-x**1.125455) - \
            2*Tc12*1.12455*x**1.12455

        # Density reducing value, Eq 5
        b = 0.8978069
        rhoc1m = IAPWS95.rhoc/(IAPWS95.M/1000)
        rhoc2m = NH3.rhoc/(NH3.M/1000)
        M = (1-x)*IAPWS95.M + x*NH3.M

        rhoc12 = 1/(1.2395117/2*(1/rhoc1m + 1/rhoc2m))
        rhonm = 1/((1-x)**2/rhoc1m + x**2/rhoc2m
                   + 2*x*(1-x**b)/rhoc12)
        rhon = rhonm*M/1000

        # sympy.diff("1/((1-x)^2/rhoc1 + x^2/rhoc2 + 2*x*(1-x^b)/rhoc12)", "x")
        # Out[]: (2*b*x**b/rhoc12 - 2*x/rhoc2 - 2*(1 - x**b)/rhoc12 -
                # (2*x - 2)/rhoc1)/(x**2/rhoc2 + 2*x*(1 - x**b)/rhoc12 +
                # (1 - x)**2/rhoc1)**2
        drhonx = M/1000*(2*b*x**b/rhoc12 - 2*(1-x**b)/rhoc12
                         - 2*x/rhoc2m + 2*(1-x)/rhoc1m)/(
                         2*x*(1-x**b)/rhoc12 + x**2/rhoc2m
                         + (1-x)**2/rhoc1m)**2

        tau = Tn/T
        delta = rho/rhon

        water = IAPWS95()
        phi1 = water._phir(tau, delta)

        ammonia = NH3()
        phi2 = ammonia._phir(tau, delta)

        Dphi = self._Dphir(tau, delta, x)

        prop = {}
        prop["tau"] = tau
        prop["delta"] = delta
        prop["fir"] = (1-x)*phi1["fir"] + x*phi2["fir"] + Dphi["fir"]
        prop["firt"] = (1-x)*phi1["firt"] + x*phi2["firt"] + Dphi["firt"]
        prop["firtt"] = (1-x)*phi1["firtt"] + x*phi2["firtt"] + Dphi["firtt"]
        prop["fird"] = (1-x)*phi1["fird"] + x*phi2["fird"] + Dphi["fird"]
        prop["firdd"] = (1-x)*phi1["firdd"] + x*phi2["firdd"] + Dphi["firdd"]
        prop["firdt"] = (1-x)*phi1["firdt"] + x*phi2["firdt"] + Dphi["firdt"]
        prop["firx"] = -phi1["fir"] + phi2["fir"] + Dphi["firx"]
        prop["F"] = prop["firx"] - delta/rhon*drhonx*prop["fird"] + \
            tau/Tn*dTnx*prop["firt"]
        return prop

    def _Dphir(self, tau, delta, x):
        """Departure function to the residual contribution to the free
        Helmholtz energy

        Parameters
        ----------
        tau : float
            Adimensional temperature, [-]
        delta : float
            Adimensional density, [-]
        x : float
            Mole fraction of ammonia in mixture, [mol/mol]

        Returns
        -------
        prop : dict
            Dictionary with departure contribution to the residual adimensional
            helmholtz energy and derivatives:

                * fir  [-]
                * firt: [∂Δfir/∂τ]δ,x  [-]
                * fird: [∂Δfir/∂δ]τ,x  [-]
                * firtt: [∂²Δfir/∂τ²]δ,x  [-]
                * firdt: [∂²Δfir/∂τ∂δ]x  [-]
                * firdd: [∂²Δfir/∂δ²]τ,x  [-]
                * firx: [∂Δfir/∂x]τ,δ  [-]

        References
        ----------
        IAPWS, Guideline on the IAPWS Formulation 2001 for the Thermodynamic
        Properties of Ammonia-Water Mixtures,
        http://www.iapws.org/relguide/nh3h2o.pdf, Eq 8
        """
        fx = x*(1-x**0.5248379)
        dfx = 1-1.5248379*x**0.5248379

        # Polinomial terms
        n = -1.855822e-2
        t = 1.5
        d = 4
        fir = n*delta**d*tau**t
        fird = n*d*delta**(d-1)*tau**t
        firdd = n*d*(d-1)*delta**(d-2)*tau**t
        firt = n*t*delta**d*tau**(t-1)
        firtt = n*t*(t-1)*delta**d*tau**(t-2)
        firdt = n*t*d*delta**(d-1)*tau**(t-1)
        firx = dfx*n*delta**d*tau**t

        # Exponential terms
        nr2 = [5.258010e-2, 3.552874e-10, 5.451379e-6, -5.998546e-13,
               -3.687808e-6]
        t2 = [0.5, 6.5, 1.75, 15, 6]
        d2 = [5, 15, 12, 12, 15]
        c2 = [1, 1, 1, 1, 2]
        for n, d, t, c in zip(nr2, d2, t2, c2):
            fir += n*delta**d*tau**t*exp(-delta**c)
            fird += n*exp(-delta**c)*delta**(d-1)*tau**t*(d-c*delta**c)
            firdd += n*exp(-delta**c)*delta**(d-2)*tau**t * \
                ((d-c*delta**c)*(d-1-c*delta**c)-c**2*delta**c)
            firt += n*t*delta**d*tau**(t-1)*exp(-delta**c)
            firtt += n*t*(t-1)*delta**d*tau**(t-2)*exp(-delta**c)
            firdt += n*t*delta**(d-1)*tau**(t-1)*(d-c*delta**c)*exp(
                -delta**c)
            firx += dfx*n*delta**d*tau**t*exp(-delta**c)

        # Exponential terms with composition
        nr3 = [0.2586192, -1.368072e-8, 1.226146e-2, -7.181443e-2, 9.970849e-2,
               1.0584086e-3, -0.1963687]
        t3 = [-1, 4, 3.5, 0, -1, 8, 7.5]
        d3 = [4, 15, 4, 5, 6, 10, 6]
        c3 = [1, 1, 1, 1, 2, 2, 2]
        for n, d, t, c in zip(nr3, d3, t3, c3):
            fir += x*n*delta**d*tau**t*exp(-delta**c)
            fird += x*n*exp(-delta**c)*delta**(d-1)*tau**t*(d-c*delta**c)
            firdd += x*n*exp(-delta**c)*delta**(d-2)*tau**t * \
                ((d-c*delta**c)*(d-1-c*delta**c)-c**2*delta**c)
            firt += x*n*t*delta**d*tau**(t-1)*exp(-delta**c)
            firtt += x*n*t*(t-1)*delta**d*tau**(t-2)*exp(-delta**c)
            firdt += x*n*t*delta**(d-1)*tau**(t-1)*(d-c*delta**c)*exp(
                -delta**c)
            firx += x*dfx*n*delta**d*tau**t*exp(-delta**c)

        n = -0.7777897
        t = 4
        d = 2
        c = 2
        fir += x**2*n*delta**d*tau**t*exp(-delta**c)
        fird += x**2*n*exp(-delta**c)*delta**(d-1)*tau**t*(d-c*delta**c)
        firdd += x**2*n*exp(-delta**c)*delta**(d-2)*tau**t * \
            ((d-c*delta**c)*(d-1-c*delta**c)-c**2*delta**c)
        firt += x**2*n*t*delta**d*tau**(t-1)*exp(-delta**c)
        firtt += x**2*n*t*(t-1)*delta**d*tau**(t-2)*exp(-delta**c)
        firdt += x**2*n*t*delta**(d-1)*tau**(t-1)*(d-c*delta**c)*exp(
            -delta**c)
        firx += x**2*dfx*n*delta**d*tau**t*exp(-delta**c)

        prop = {}
        prop["fir"] = fir*fx
        prop["firt"] = firt*fx
        prop["firtt"] = firtt*fx
        prop["fird"] = fird*fx
        prop["firdd"] = firdd*fx
        prop["firdt"] = firdt*fx
        prop["firx"] = firx
        return prop


def Ttr(x):
    """Equation for the triple point of ammonia-water mixture

    Parameters
    ----------
    x : float
        Mole fraction of ammonia in mixture, [mol/mol]

    Returns
    -------
    Ttr : float
        Triple point temperature, [K]

    Notes
    -----
    Raise :class:`NotImplementedError` if input isn't in limit:

        * 0 ≤ x ≤ 1

    References
    ----------
    IAPWS, Guideline on the IAPWS Formulation 2001 for the Thermodynamic
    Properties of Ammonia-Water Mixtures,
    http://www.iapws.org/relguide/nh3h2o.pdf, Eq 9
    """
    if 0 <= x <= 0.33367:
        Ttr = 273.16*(1-0.3439823*x-1.3274271*x**2-274.973*x**3)
    elif 0.33367 < x <= 0.58396:
        Ttr = 193.549*(1-4.987368*(x-0.5)**2)
    elif 0.58396 < x <= 0.81473:
        Ttr = 194.38*(1-4.886151*(x-2/3)**2+10.37298*(x-2/3)**3)
    elif 0.81473 < x <= 1:
        Ttr = 195.495*(1-0.323998*(1-x)-15.87560*(1-x)**4)
    else:
        raise NotImplementedError("Incoming out of bound")
    return Ttr



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""
Miscelaneous internal utilities. This module include:

    * :func:`getphase`: Get phase string of state
    * :class:`_fase`: Base class to define a phase state
    * :func:`deriv_H`: Calculate generic partial derivative with a fundamental
      Helmholtz free energy equation of state
    * :func:`deriv_G`: Calculate generic partial derivative with a fundamental
      Gibbs free energy equation of state
"""

from __future__ import division


def getphase(Tc, Pc, T, P, x, region):
    """Return fluid phase string name

    Parameters
    ----------
    Tc : float
        Critical temperature, [K]
    Pc : float
        Critical pressure, [MPa]
    T : float
        Temperature, [K]
    P : float
        Pressure, [MPa]
    x : float
        Quality, [-]
    region: int
        Region number, used only for IAPWS97 region definition

    Returns
    -------
    phase : str
        Phase name
    """
    # Avoid round problem
    P = round(P, 8)
    T = round(T, 8)
    if P > Pc and T > Tc:
        phase = "Supercritical fluid"
    elif T > Tc:
        phase = "Gas"
    elif P > Pc:
        phase = "Compressible liquid"
    elif P == Pc and T == Tc:
        phase = "Critical point"
    elif region == 4 and x == 1:
        phase = "Saturated vapor"
    elif region == 4 and x == 0:
        phase = "Saturated liquid"
    elif region == 4:
        phase = "Two phases"
    elif x == 1:
        phase = "Vapour"
    elif x == 0:
        phase = "Liquid"
    return phase


class _fase(object):
    """Class to implement a null phase"""

    v = None
    rho = None

    h = None
    s = None
    u = None
    a = None
    g = None

    cp = None
    cv = None
    cp_cv = None
    w = None
    Z = None
    fi = None
    f = None

    mu = None
    k = None
    nu = None
    Prandt = None
    epsilon = None
    alfa = None
    n = None

    alfap = None
    betap = None
    joule = None
    Gruneisen = None
    alfav = None
    kappa = None
    betas = None
    gamma = None
    Kt = None
    kt = None
    Ks = None
    ks = None
    dpdT_rho = None
    dpdrho_T = None
    drhodT_P = None
    drhodP_T = None
    dhdT_rho = None
    dhdT_P = None
    dhdrho_T = None
    dhdrho_P = None
    dhdP_T = None
    dhdP_rho = None

    Z_rho = None
    IntP = None
    hInput = None


def deriv_H(state, z, x, y, fase):
    r"""Calculate generic partial derivative
    :math:`\left.\frac{\partial z}{\partial x}\right|_{y}` from a fundamental
    helmholtz free energy equation of state

    Parameters
    ----------
    state : any python object
        Only need to define P and T properties, non phase specific properties
    z : str
        Name of variables in numerator term of derivatives
    x : str
        Name of variables in denominator term of derivatives
    y : str
        Name of constant variable in partial derivaritive
    fase : any python object
        Define phase specific properties (v, cv, alfap, s, betap)

    Notes
    -----
    x, y and z can be the following values:

        * P: Pressure
        * T: Temperature
        * v: Specific volume
        * rho: Density
        * u: Internal Energy
        * h: Enthalpy
        * s: Entropy
        * g: Gibbs free energy
        * a: Helmholtz free energy

    Returns
    -------
    deriv : float
        ∂z/∂x|y

    References
    ----------
    IAPWS, Revised Advisory Note No. 3: Thermodynamic Derivatives from IAPWS
    Formulations, http://www.iapws.org/relguide/Advise3.pdf
    """
    # We use the relation between rho and v and his partial derivative
    # ∂v/∂b|c = -1/ρ² ∂ρ/∂b|c
    # ∂a/∂v|c = -ρ² ∂a/∂ρ|c
    mul = 1
    if z == "rho":
        mul = -fase.rho**2
        z = "v"
    if x == "rho":
        mul = -1/fase.rho**2
        x = "v"
    if y == "rho":
        y = "v"

    if x == "P":
        dTdx = state.P*1000*fase.alfap
        dvdx = -state.P*1000*fase.betap
    elif x == "T":
        dTdx = 1
        dvdx = 0
    elif x == "v":
        dTdx = 0
        dvdx = 1
    elif x == "u":
        dTdx = fase.cv
        dvdx = state.P*1000*(state.T*fase.alfap-1)
    elif x == "h":
        dTdx = fase.cv+state.P*1000*fase.v*fase.alfap
        dvdx = state.P*1000*(state.T*fase.alfap-fase.v*fase.betap)
    elif x == "s":
        dTdx = fase.cv/state.T
        dvdx = state.P*1000*fase.alfap
    elif x == "g":
        dTdx = state.P*1000*fase.v*fase.alfap-fase.s
        dvdx = -state.P*1000*fase.v*fase.betap
    elif x == "a":
        dTdx = -fase.s
        dvdx = -state.P*1000

    if y == "P":
        dTdy = state.P*1000*fase.alfap
        dvdy = -state.P*1000*fase.betap
    elif y == "T":
        dTdy = 1
        dvdy = 0
    elif y == "v":
        dTdy = 0
        dvdy = 1
    elif y == "u":
        dTdy = fase.cv
        dvdy = state.P*1000*(state.T*fase.alfap-1)
    elif y == "h":
        dTdy = fase.cv+state.P*1000*fase.v*fase.alfap
        dvdy = state.P*1000*(state.T*fase.alfap-fase.v*fase.betap)
    elif y == "s":
        dTdy = fase.cv/state.T
        dvdy = state.P*1000*fase.alfap
    elif y == "g":
        dTdy = state.P*1000*fase.v*fase.alfap-fase.s
        dvdy = -state.P*1000*fase.v*fase.betap
    elif y == "a":
        dTdy = -fase.s
        dvdy = -state.P*1000

    if z == "P":
        dTdz = state.P*1000*fase.alfap
        dvdz = -state.P*1000*fase.betap
    elif z == "T":
        dTdz = 1
        dvdz = 0
    elif z == "v":
        dTdz = 0
        dvdz = 1
    elif z == "u":
        dTdz = fase.cv
        dvdz = state.P*1000*(state.T*fase.alfap-1)
    elif z == "h":
        dTdz = fase.cv+state.P*1000*fase.v*fase.alfap
        dvdz = state.P*1000*(state.T*fase.alfap-fase.v*fase.betap)
    elif z == "s":
        dTdz = fase.cv/state.T
        dvdz = state.P*1000*fase.alfap
    elif z == "g":
        dTdz = state.P*1000*fase.v*fase.alfap-fase.s
        dvdz = -state.P*1000*fase.v*fase.betap
    elif z == "a":
        dTdz = -fase.s
        dvdz = -state.P*1000

    deriv = (dvdz*dTdy-dTdz*dvdy)/(dvdx*dTdy-dTdx*dvdy)
    return mul*deriv


def deriv_G(state, z, x, y, fase):
    r"""Calculate generic partial derivative
    :math:`\left.\frac{\partial z}{\partial x}\right|_{y}` from a fundamental
    Gibbs free energy equation of state

    Parameters
    ----------
    state : any python object
        Only need to define P and T properties, non phase specific properties
    z : str
        Name of variables in numerator term of derivatives
    x : str
        Name of variables in denominator term of derivatives
    y : str
        Name of constant variable in partial derivaritive
    fase : any python object
        Define phase specific properties (v, cp, alfav, s, xkappa)

    Notes
    -----
    x, y and z can be the following values:

        * P: Pressure
        * T: Temperature
        * v: Specific volume
        * rho: Density
        * u: Internal Energy
        * h: Enthalpy
        * s: Entropy
        * g: Gibbs free energy
        * a: Helmholtz free energy

    Returns
    -------
    deriv : float
        ∂z/∂x|y

    References
    ----------
    IAPWS, Revised Advisory Note No. 3: Thermodynamic Derivatives from IAPWS
    Formulations, http://www.iapws.org/relguide/Advise3.pdf
    """
    mul = 1
    if z == "rho":
        mul = -fase.rho**2
        z = "v"
    if x == "rho":
        mul = -1/fase.rho**2
        x = "v"

    if x == "P":
        dPdx = 1.0
        dTdx = 0.0
    elif x == "T":
        dPdx = 0.0
        dTdx = 1.0
    elif x == "v":
        dPdx = -fase.v*fase.xkappa
        dTdx = fase.v*fase.alfav
    elif x == "u":
        dPdx = fase.v*(state.P*1000.0*fase.xkappa-state.T*fase.alfav),
        dTdx = fase.cp-state.P*1000.0*fase.v*fase.alfav
    elif x == "h":
        dPdx = fase.v*(1.0-state.T*fase.alfav)
        dTdx = fase.cp
    elif x == "s":
        dPdx = -fase.v * fase.alfav
        dTdx = fase.cp/state.T
    elif x == "g":
        dPdx = fase.v
        dTdx = -fase.s
    elif x == "a":
        dPdx = state.P*1000.0*fase.v*fase.xkappa
        dTdx = -state.P * 1000.0 * fase.v * fase.alfav - fase.s
    else:
        raise ValueError("x must be one of P, T, v, u, h, s, g, a")

    if y == "P":
        dPdy = 1.0
        dTdy = 0.0
    elif y == "T":
        dPdy = 0.0
        dTdy = 1.0
    elif y == "v":
        dPdy = -fase.v*fase.xkappa
        dTdy = fase.v*fase.alfav
    elif y == "u":
        dPdy = fase.v*(state.P*1000.0*fase.xkappa-state.T*fase.alfav),
        dTdy = fase.cp-state.P*1000.0*fase.v*fase.alfav
    elif y == "h":
        dPdy = fase.v*(1.0-state.T*fase.alfav)
        dTdy = fase.cp
    elif y == "s":
        dPdy = -fase.v * fase.alfav
        dTdy = fase.cp/state.T
    elif y == "g":
        dPdy = fase.v
        dTdy = -fase.s
    elif y == "a":
        dPdy = state.P*1000.0*fase.v*fase.xkappa
        dTdy = -state.P * 1000.0 * fase.v * fase.alfav - fase.s
    else:
        raise ValueError("y must be one of P, T, v, u, h, s, g, a")

    if z == "P":
        dPdz = 1.0
        dTdz = 0.0
    elif z == "T":
        dPdz = 0.0
        dTdz = 1.0
    elif z == "v":
        dPdz = -fase.v*fase.xkappa
        dTdz = fase.v*fase.alfav
    elif z == "u":
        dPdz = fase.v*(state.P*1000.0*fase.xkappa-state.T*fase.alfav),
        dTdz = fase.cp-state.P*1000.0*fase.v*fase.alfav
    elif z == "h":
        dPdz = fase.v*(1.0-state.T*fase.alfav)
        dTdz = fase.cp
    elif z == "s":
        dPdz = -fase.v * fase.alfav
        dTdz = fase.cp/state.T
    elif z == "g":
        dPdz = fase.v
        dTdz = -fase.s
    elif z == "a":
        dPdz = state.P*1000.0*fase.v*fase.xkappa
        dTdz = -state.P * 1000.0 * fase.v * fase.alfav - fase.s
    else:
        raise ValueError("z must be one of P, T, v, u, h, s, g, a")

    deriv = (dPdz * dTdy - dTdz * dPdy) / (dPdx * dTdy - dTdx * dPdy)
    return mul*deriv



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Install IAPWS module using setuptools."""

from setuptools import setup

import io  # for backwards compatibility with Python 2
import os


with open(os.path.join("iapws", "VERSION")) as version_file:
    __version__ = version_file.read().strip()

with io.open('README.rst', encoding="utf8") as file:
    long_description = file.read()

setup(
    name='iapws',
    version=__version__,
    packages=['iapws'],
    include_package_data=True,
    author='jjgomera',
    author_email='jjgomera@gmail.com',
    url='https://github.com/jjgomera/iapws',
    download_url='https://github.com/jjgomera/iapws/tarball/v' + __version__,
    description='Python implementation of standards from The International'
                'Association for the Properties of Water and Steam',
    long_description=long_description,
    license="gpl v3",
    python_requires='>=2.7,!=3.0.*,!=3.1.*,!=3.2.*,!=3.3.*',
    install_requires=["scipy>=1.2"],
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Education",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Topic :: Scientific/Engineering",
        "Topic :: Scientific/Engineering :: Chemistry",
        "Topic :: Scientific/Engineering :: Physics",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
)
